/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#ifndef __vtkVRPN_CAMERA_MANIPULATOR_H__
#define __vtkVRPN_CAMERA_MANIPULATOR_H__

#include "vtkVRPNObject.h"

#include <vtkCommand.h>
#include <vtkSetGet.h>
#include <quat.h>

//BTX
class vtkObject;
class vtkVRPNClient;
class vtkCamera;
class vtkTransform;
//ETX

// Handles TRACKER and ANALOG events to manipulate a camera.
class VTK_COMMON_EXPORT vtkVRPNCameraManipulator : public vtkVRPNObject
{
public:
  vtkTypeRevisionMacro(vtkVRPNCameraManipulator,vtkVRPNObject);
  static vtkVRPNCameraManipulator* New();
  ~vtkVRPNCameraManipulator();

  vtkSetMacro(TrackerSensor,int);
  vtkGetMacro(TrackerSensor,int);

  // Set/Get Analog channels to use for translations on x,y and z axes.
  vtkSetVectorMacro(AnalogTranslateChannels,int,3);
  vtkGetVectorMacro(AnalogTranslateChannels,int,3);

  // Set/Get Analog channels to use for rotations around x,y and z axes.
  vtkSetVectorMacro(AnalogRotateChannels,int,3);
  vtkGetVectorMacro(AnalogRotateChannels,int,3);

  // Set/Get the manipulated camera.
  vtkGetMacro(Camera,vtkCamera*);
  void SetCamera(vtkCamera* cam);

  // Resets internal parameters
  void ResetCamera();

  // Set/Get view volume bounds.
  void SetBounds(const double b[6]);
  vtkGetVectorMacro(Bounds,double,6);

 protected:
  vtkVRPNCameraManipulator();  
  void ProcessEvent(vtkObject *caller, unsigned long eventId, void *callData);

 private:
  double Bounds[6];
  double _focal[3], _position[3], _viewUp[3];
  double _startPosition[3];
  q_type _startOrient;
  double _scale;
  vtkCamera* Camera;
  vtkTransform* _camTransform;
  int AnalogTranslateChannels[3];
  int AnalogRotateChannels[3];
  int TrackerSensor;
  bool _resetPosition;
};

#endif
