/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#include "vtkVRPNClient.h"

#include <vtkRenderWindow.h>
#include <vtkCommand.h>
#include <vtkCallbackCommand.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkInteractorStyle.h>
#include <vtkObjectFactory.h>

#include <string>
#include <map>
using namespace std;

vtkCxxRevisionMacro(vtkVRPNClient,"1.0");
vtkStandardNewMacro(vtkVRPNClient);

vtkVRPNClient::vtkVRPNClient()
  : Enabled(false)
{
}

void vtkVRPNClient::Enable()
{
  this->Enabled = true;
}

void vtkVRPNClient::Disable()
{
  this->Enabled = false;
}

void vtkVRPNClient::ProcessEvent(vtkObject* caller, unsigned long eventId, void *callData )
{
  if( this->Enabled && eventId==this->UpdateEvent )
  {
    this->MainLoop();
  }
}

void vtkVRPNClient::MainLoop()
{
  // parse trackers, buttons etc. until everything's updated
#define VrpnDeviceEnum(device) \
  for(device##Map::iterator it=device##s.begin(); it!=device##s.end(); it++) it->second->MainLoop();
#include "vtk_vrpn_database.h"
}

// Implementation of AddDevice methods
#define VrpnDeviceEnum(device)                              \
void vtkVRPNClient::Add##device ( const std::string& _name ) \
{                                                           \
  string name = _name;                                      \
  if(name.find('@')==string::npos) name+="@localhost";      \
  if( device##s.find(name) != device##s.end() )             \
  {                                                         \
    this->UnRegister( device##s[name] );                    \
  }              vtkVRPNClient::                                           \
  device##s[name] = vtkVRPN##device::New();                 \
  device##s[name]->ConnectTo( name );                       \
  this->Register( device##s[name] );                        \
}
#include "vtk_vrpn_database.h"


// Implementation of GetDevice methods
#define VrpnDeviceEnum(device)                                        \
vtkVRPN##device* vtkVRPNClient::Get##device( const std::string& name) \
{                                                                     \
  device##Map::iterator it;                                           \
  if( name=="" )                                                      \
  {                                                                    \
    it = device##s.begin();                                           \
  }                                                                   \
  else                                                                \
  {                                                                 \
    if(name.find('@')!=string::npos) { it = device##s.find(name); }       \
    else { it = device##s.find(name+"@localhost"); }                      \
  }                                                                   \
  if( it!=device##s.end() ){ return it->second; }                        \
  else { return 0;  }                                                      \
}
#include "vtk_vrpn_database.h"

