/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#ifndef __VTK_VRPN_CLIENT_H
#define __VTK_VRPN_CLIENT_H

#include "vtkVRPNObject.h"
#include "vtkVRPNAnalog.h"
#include "vtkVRPNAnalog_Output.h"
#include "vtkVRPNButton.h"
#include "vtkVRPNDial.h"
#include "vtkVRPNForceDevice.h"
#include "vtkVRPNPoser.h"
#include "vtkVRPNSound.h"
#include "vtkVRPNTracker.h"

#include <vtkObject.h>
#include <vtkSetGet.h>
#include <string>
#include <list>
#include <map>

class vtkRenderWindow;
class vtkCommand;
class vtkCallbackCommand;

class VTK_COMMON_EXPORT vtkVRPNClient : public vtkVRPNObject
{
 public:
  vtkTypeRevisionMacro(vtkVRPNClient, vtkVRPNObject);
  static vtkVRPNClient* New();

  // Create and manage a new remote, given a VRPN remote name.
  // VRPN's remote device names have the form : DeviceName@remote.host.address
  // where DeviceName is the name declared by the server.
#define VrpnDeviceEnum(device) \
  void              Add##device ( const std::string& name    );
#include "vtk_vrpn_database.h"

  // Retreive a created device.
  // If @host is ommited, @localhost is assumed.
  // if no name at all is given, the first found device is returned.
  // if no device has been found, NULL is returned.
#define VrpnDeviceEnum(device) \
  vtkVRPN##device * Get##device ( const std::string& name="" );
#include "vtk_vrpn_database.h"

  // enable or disable vrpn remote objects' update
  void Enable();
  void Disable();

  // Set/Get event update is triggered on.
  vtkSetMacro(UpdateEvent,unsigned long);
  vtkGetMacro(UpdateEvent,unsigned long);

 protected:
  // Constructor
  vtkVRPNClient();

  // calls the mainloop() method on each device.
  void MainLoop();

  // process incoming events
  void ProcessEvent(vtkObject* caller, unsigned long eventId, void *callData );

 private:
#define VrpnDeviceEnum(device) \
  typedef std::map< std::string , vtkVRPN##device * > device##Map; \
  device##Map device##s;
#include "vtk_vrpn_database.h"

  unsigned long UpdateEvent;
  bool Enabled;
};

#endif
