/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#ifndef __VTKVRPN_FORCEDEVICE_H
#define __VTKVRPN_FORCEDEVICE_H

#include "vtkVRPNRemoteObject.h"
#include <vrpn_ForceDevice.h>
#include <vtkSetGet.h>

// derived class that fixes uniformity of class vrpn_ForceDevice_Remote.
// it defines methods called register_change_handler like all other devices have.
class vrpn_ForceDevice_Remote_fixed : public vrpn_ForceDevice_Remote
{ 
 public:
  inline vrpn_ForceDevice_Remote_fixed(const char *name,vrpn_Connection* c=0)
    : vrpn_ForceDevice_Remote(name,c) {}

  inline void register_change_handler(void* ud,vrpn_FORCECHANGEHANDLER handler)
    { this->register_change_handler(ud,handler); }
  inline void unregister_change_handler(void* ud,vrpn_FORCECHANGEHANDLER handler)
    { this->unregister_change_handler(ud,handler); }

  inline void register_change_handler(void* ud,vrpn_FORCESCPHANDLER handler)
    { this->register_scp_change_handler(ud,handler); }
  inline void unregister_change_handler(void* ud,vrpn_FORCESCPHANDLER handler)
    { this->unregister_scp_change_handler(ud,handler); }
};

typedef vtkVRPNRemoteObject<vrpn_ForceDevice_Remote_fixed> vtkVRPNRemoteObject_vrpn_ForceDevice_Remote;

class VTK_COMMON_EXPORT vtkVRPNForceDevice : public vtkVRPNRemoteObject_vrpn_ForceDevice_Remote
{
public:
	vtkTypeRevisionMacro(vtkVRPNForceDevice,vtkVRPNRemoteObject_vrpn_ForceDevice_Remote);
        static vtkVRPNForceDevice* New();

        void RegisterHandlers();
        void UnRegisterHandlers();
};

#endif

