/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#include "vtkVRPNObject.h"
#include "vtkVRPNCommand.h"
#include "vtkVRPNAnalog.h"
#include "vtkVRPNButton.h"
#include "vtkVRPNDial.h"
#include "vtkVRPNForceDevice.h"
#include "vtkVRPNTracker.h"

#include <vtkCallbackCommand.h>
#include <vtkObjectFactory.h>

vtkCxxRevisionMacro(vtkVRPNObject,"1.0");
vtkStandardNewMacro(vtkVRPNObject);

vtkVRPNObject::vtkVRPNObject()
{
  vtkCallbackCommand* callback = vtkCallbackCommand::New();
  callback->SetClientData(this);
  callback->SetCallback( &vtkVRPNObject::callback );
  Observer = callback;
}

// canvas for writting your own event processor
void vtkVRPNObject::ProcessEvent(vtkObject* caller, unsigned long eventId, void *callData)
{
  switch(eventId)
    {
      case vtkVRPNCommand::VRPN_TRACKER :
	{
	  vrpn_TRACKERCB* info = (vrpn_TRACKERCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_TRACKERVEL :
	{
	  vrpn_TRACKERVELCB* info = (vrpn_TRACKERVELCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_TRACKERACC :
	{
	  vrpn_TRACKERACCCB* info = (vrpn_TRACKERACCCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_TRACKERTRACKER2ROOM :
	{
	  vrpn_TRACKERTRACKER2ROOMCB* info = (vrpn_TRACKERTRACKER2ROOMCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_TRACKERUNIT2SENSOR:
	{
	  vrpn_TRACKERUNIT2SENSORCB* info = (vrpn_TRACKERUNIT2SENSORCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_TRACKERWORKSPACE:
	{
	  vrpn_TRACKERWORKSPACECB* info = (vrpn_TRACKERWORKSPACECB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_BUTTON:
	{
	  vrpn_BUTTONCB* info = (vrpn_BUTTONCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_ANALOG:
	{
	  vrpn_ANALOGCB* info = (vrpn_ANALOGCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_DIAL:
	{
	  vrpn_DIALCB* info = (vrpn_DIALCB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_FORCE:
	{
	  vrpn_FORCECB* info = (vrpn_FORCECB*) callData;
	}
	break;
      case vtkVRPNCommand::VRPN_FORCESCP:
	{
	  vrpn_FORCESCPCB* info = (vrpn_FORCESCPCB*) callData;
	}
	break;
    }
}

void vtkVRPNObject::callback(vtkObject *caller, unsigned long eid, void *clientdata, void *calldata)
{
	vtkVRPNObject* self = (vtkVRPNObject*) clientdata;
	self->ProcessEvent(caller,eid,calldata);
}
