/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#ifndef __VTKVRPN_REMOTE_OBJECT_H
#define __VTKVRPN_REMOTE_OBJECT_H

#include "vtkVRPNObject.h"

#include <vtkObject.h>
#include <vtkSetGet.h>
#include <string>

template<class RemoteObj>
class VTK_COMMON_EXPORT vtkVRPNRemoteObject : public vtkVRPNObject
{
 public:
  vtkTypeRevisionMacro(vtkVRPNRemoteObject,vtkVRPNObject);
  static vtkVRPNRemoteObject* New();

  ~vtkVRPNRemoteObject();

  // Connect remote device client to the server end.
  // name should be like "DeviceName@server.host"
  void ConnectTo(const std::string& name);

  // Returns internal vrpn object (i.e. vrpn_Tracker_Remote).
  vtkGetMacro(VRPNObject,RemoteObj*);

  // should be called only by vtkVRPNClient.
  // trigers vrpn update.
  void MainLoop();

 protected:
  vtkVRPNRemoteObject();
  virtual void RegisterHandlers();
  virtual void UnRegisterHandlers();

 private:
  RemoteObj * VRPNObject;
  template<int event, class infoT> static inline void handle(void* calldata, const infoT datacb)
  {
	vtkObject* obj = (vtkObject*)calldata;
	infoT copy = datacb;
	obj->InvokeEvent( event, (void*)&copy );
  }
};

#endif /* __VTKVRPN_REMOTE_OBJECT_H */
