/*=========================================================================
  
  Authors: Daniel Aguilera, Thierry Carrard, Claire Guilbaud
  
  Copyright (c) CEA/DIF (Comissariat a l'Energie Atomique, Direction Ile de France) 
                BP12 91680 Bruyeres Le Chatel 
                France 
  All rights reserved.
  
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.
  
=========================================================================*/


#ifndef __WAND_MANIPULATOR_H__
#define __WAND_MANIPULATOR_H__

#include "vtkVRPNObject.h"

#include <vtkObject.h>
#include <vtkCommand.h>
#include <vtkSetGet.h>

class vtkTransform;
class vtkRenderWindow;
class vtkProp3D;
class vtkConeSource;
class vtkActor;

class VTK_COMMON_EXPORT vtkVRPNWandManipulator : public vtkVRPNObject
{
public:
  vtkTypeRevisionMacro(vtkVRPNWandManipulator,vtkVRPNObject);
  static vtkVRPNWandManipulator* New();
  ~vtkVRPNWandManipulator();

  void SetRenderWindow(vtkRenderWindow* rw);
  void SetProp3D(vtkProp3D* p);

  vtkSetMacro(TrackerSensor,int);
  vtkGetMacro(TrackerSensor,int);

  vtkSetMacro(Button,int);
  vtkGetMacro(Button,int);

  vtkSetMacro(MoveThreshold,double);
  vtkGetMacro(MoveThreshold,double);


 protected:
  vtkVRPNWandManipulator();
  void ProcessEvent(vtkObject* caller, unsigned long eventId, void *callData );
  void Move();
  void DragStart();
  void DragEnd();

  enum WandState
  {
    FREE_MOVE,
    DRAG_START,
    DRAG_PAUSE,
    DRAG_MOVE
  };

 protected:
  double _wandMatrix[16]; // wand matrix
  double _objMatrix[16]; // object's matrix
  double _invStartWandMatrix[16]; // inverse of wand matrix at drag start
  double _invPauseWandMatrix[16]; // inverse of wand matrix at last pause

  double MoveThreshold;
  double _scale;
  WandState _state;
  int TrackerSensor;
  int Button;

  vtkTransform* _objTransform;
  vtkTransform* _wandTransform;

  vtkRenderWindow* _renwin;
  vtkProp3D *_prop;

  vtkConeSource* _cone;
  vtkActor* _coneActor;
};

#endif
